'use strict';

var fs = require('fs');
var http = require('http');
var path = require('path');

module.exports = {
  port: 8088,
  basePath: '/',
  serve: function(cb) {
    module.exports._server = http.createServer(function(req, res) {
      // Function to check if the requested path goes outside of allowed base path.
      function verifyPath(reqpath) {
        var tmp = module.exports.basePath;
        if (path.isAbsolute(reqpath)) {
          tmp = path;
        } else {
          tmp = path.join(module.exports.basePath, reqpath);
        }

        if (tmp.substr(0, module.exports.basePath.length) !== module.exports.basePath) return false;
        return true;
      }

      function serverError(e) {
        res.statusCode = 500;
        res.end(e.message);
      }

      function badRequest() {
        res.statusCode = 400;
        res.end('Error with IO request.');
      }

      function forbidden() {
        res.statusCode = 403;
        res.end('Error, access denied.');
      }

      var data = '';

      req.on('data', function(chunk) {
        data += chunk;
      });

      req.on('end', function() {
        try {
          var reqType = req.headers['tmpdisk-request-type'];
          if (!reqType) return badRequest();
          switch(reqType) {
            case 'read':
              if (!verifyPath(data)) return forbidden();
              fs.stat(path.join(module.exports.basePath, data), function(err, stats) {
                if (err) return serverError(err);
                if (stats.isDirectory()) {
                  res.setHeader('Tmpdisk-Response-Is-Directory', 'true');
                  fs.readdir(path.join(module.exports.basePath, data), function(err, files) {
                    if (err) return serverError(err);
                    res.statusCode = 200;
                    res.end(JSON.stringify(files));
                  });
                } else {
                  res.setHeader('Tmpdisk-Response-Is-Directory', 'false');
                  var stream = fs.createReadStream(path.join(module.exports.basePath, data));
                  stream.pipe(res);
                }
              });
              break;
            case 'write':
              var reqPath = req.headers['tmpdisk-request-file-path'];
              if (!reqPath) return badRequest();
              if (!verifyPath(reqPath)) return forbidden();
              fs.stat(path.join(module.exports.basePath, reqPath), function(err, stats) {
                if (err) {
                  // the error usually means that the file doesn't exist
                  return fs.writeFile(path.join(module.exports.basePath, reqPath), data, req.headers['tmpdisk-request-file-encoding'], function(err) {
                    if (err) return serverError(err);
                    res.statusCode = 200;
                    res.end('200 ok.');
                  });
                }
                if (stats.isDirectory()) {
                  badRequest();
                } else {
                  fs.writeFile(path.join(module.exports.basePath, reqPath), data, req.headers['tmpdisk-request-file-encoding'], function(err) {
                    if (err) return serverError(err);
                    res.statusCode = 200;
                    res.end('200 ok.');
                  });
                }
              });
              break;
            case 'delete':
              if (!verifyPath(data)) return forbidden();
              fs.stat(path.join(module.exports.basePath, data), function(err, stats) {
                if (err) return serverError(err);
                if (stats.isDirectory()) {
                  fs.rmdir(path.join(module.exports.basePath, data), function(err) {
                    if (err) return serverError(err);
                    res.statusCode = 200;
                    res.end('200 ok.');
                  });
                } else {
                  fs.unlink(path.join(module.exports.basePath, data), function(err) {
                    if (err) return serverError(err);
                    res.statusCode = 200;
                    res.end('200 ok.');
                  });
                }
              });
              break;
            case 'append':
              var reqPath = req.headers['tmpdisk-request-file-path'];
              if (!reqPath) return badRequest();
              if (!verifyPath(data)) return forbidden();
              fs.stat(path.join(module.exports.basePath, reqPath), function(err, stats) {
                if (err) return serverError(err);
                if (stats.isDirectory()) {
                  badRequest();
                } else {
                  fs.appendFile(path.join(module.exports.basePath, reqPath), data, req.headers['tmpdisk-request-file-encoding'], function(err) {
                    if (err) return serverError(err);
                    res.statusCode = 200;
                    res.end('200 ok.');
                  });
                }
              });
              break;
            default:
              badRequest();
              break;
          }
        } catch(e) {
          serverError(e);
        }
      });
    });
    module.exports._server.listen(module.exports.port, function() {
      if (cb) cb();
    });
  },
  close: function(cb) {
    if (!module.exports._server) {
      if (cb) {
        cb(new Error('server has not yet been started.'));
      }
    }
    module.exports._server.close(cb);
  }
}
