'use strict';

var test = require('tape');
var tmpdisk = require('../index');
var http = require('http');
var fs = require('fs');

test('can start server?', function(t) {
  t.plan(1);
  tmpdisk.port = 8089;
  tmpdisk.basePath = __dirname;
  try {
    tmpdisk.serve(function() {
      tmpdisk.close();
      t.pass('server creation success.');
    });
  } catch(e) {
    t.fail('failed to create server.');
  }
});

test('can close server?', function(t) {
  t.plan(1);
  tmpdisk.port = 8089;
  tmpdisk.basePath = __dirname;
  try {
    tmpdisk.serve(function() {
      tmpdisk.close(function(err) {
        if (err) {
          t.fail('failed to close server.');
        }
        t.pass('server close success.');
      });
    });
  } catch(e) {
    t.fail('failed to create server.');
  }
});

test('does throw error on trying to close an unopened server?', function(t) {
  t.plan(1);
  tmpdisk.port = 8089;
  tmpdisk.basePath = __dirname;
  tmpdisk.close(function(err) {
    if (err) {
      return t.pass('succeeded to throw error.');
    }
    t.fail('failed to throw error.');
  });
});

test('can access server?', function(t) {
  t.plan(1);
  tmpdisk.port = 8089;
  tmpdisk.basePath = __dirname;
  try {
    tmpdisk.serve(function() {
      var reqData = 'test.txt';
      var req = http.request({
        hostname: 'localhost',
        port: 8089,
        path: '/',
        method: 'POST',
        headers: {
          'Content-Type': 'plain/text',
          'Content-Length': reqData.length,
          'Tmpdisk-Request-Type': 'read'
        }
      }, function(res) {
        tmpdisk.close(function(err) {
          if (err) {
            t.fail('failed to close server.');
          }
          t.pass('server access success.');
        });
      });
      req.on('error', function(e) {
        t.fail('failed to access server.');
      });
      req.write(reqData);
      req.end();
    });
  } catch(e) {
    t.fail('failed to create server.');
  }
});

test('can read file?', function(t) {
  t.plan(1);
  tmpdisk.port = 8089;
  tmpdisk.basePath = __dirname;
  try {
    tmpdisk.serve(function() {
      var reqData = 'test.txt';
      var req = http.request({
        hostname: 'localhost',
        port: 8089,
        path: '/',
        method: 'POST',
        headers: {
          'Content-Type': 'plain/text',
          'Content-Length': reqData.length,
          'Tmpdisk-Request-Type': 'read'
        }
      }, function(res) {
        var data = '';

        res.on('data', function(chunk) {
          data += String(chunk);
        });

        res.on('end', function() {
          if (res.statusCode === 500 || res.statusCode === 403 || res.statusCode === 400) {
            tmpdisk.close();
            return t.fail('failed to perform server request.');
          }

          if (data !== 'this is some test text.') {
            tmpdisk.close();
            return t.fail('file contents recieved are not equal to actual contents.');
          }

          tmpdisk.close();
          t.pass('successfully read file.');
        });
      });
      req.on('error', function(e) {
        t.fail('failed to access server.');
      });
      req.write(reqData);
      req.end();
    });
  } catch(e) {
    t.fail('failed to create server.');
  }
});

test('can read folder?', function(t) {
  t.plan(1);
  tmpdisk.port = 8089;
  tmpdisk.basePath = __dirname;
  try {
    tmpdisk.serve(function() {
      var reqData = 'testFolder';
      var req = http.request({
        hostname: 'localhost',
        port: 8089,
        path: '/',
        method: 'POST',
        headers: {
          'Content-Type': 'plain/text',
          'Content-Length': reqData.length,
          'Tmpdisk-Request-Type': 'read'
        }
      }, function(res) {
        var data = '';

        res.on('data', function(chunk) {
          data += String(chunk);
        });

        res.on('end', function() {
          data = JSON.parse(data);
          if (res.statusCode === 500 || res.statusCode === 403 || res.statusCode === 400) {
            tmpdisk.close();
            return t.fail('failed to perform server request.');
          }

          if (data[0] !== 'blah.txt') {
            tmpdisk.close();
            return t.fail('folder contents recieved are not equal to actual contents.');
          }

          tmpdisk.close();
          t.pass('successfully read folder.');
        });
      });
      req.on('error', function(e) {
        t.fail('failed to access server.');
      });
      req.write(reqData);
      req.end();
    });
  } catch(e) {
    t.fail('failed to create server.');
  }
});

test('can write file?', function(t) {
  t.plan(1);
  tmpdisk.port = 8089;
  tmpdisk.basePath = __dirname;
  try {
    tmpdisk.serve(function() {
      var reqData = 'this is some *new* test text.';
      var req = http.request({
        hostname: 'localhost',
        port: 8089,
        path: '/',
        method: 'POST',
        headers: {
          'Content-Type': 'plain/text',
          'Content-Length': reqData.length,
          'Tmpdisk-Request-File-Path': 'test.txt',
          'Tmpdisk-Request-File-Encoding': 'utf8',
          'Tmpdisk-Request-Type': 'write'
        }
      }, function(res) {
        var data = '';

        res.on('data', function(chunk) {
          data += String(chunk);
        });

        res.on('end', function() {
          if (res.statusCode === 500 || res.statusCode === 403 || res.statusCode === 400) {
            tmpdisk.close();
            return t.fail('failed to perform server request.');
          }

          if (String(fs.readFileSync(__dirname + '/test.txt')) !== 'this is some *new* test text.') {
            tmpdisk.close();
            return t.fail('new file contents recieved are not equal to what they should be.');
          }

          tmpdisk.close();
          t.pass('successfully wrote file.');
        });
      });
      req.on('error', function(e) {
        t.fail('failed to access server.');
      });
      req.write(reqData);
      req.end();
    });
  } catch(e) {
    t.fail('failed to create server.');
  }
});

test('can append file?', function(t) {
  t.plan(1);
  tmpdisk.port = 8089;
  tmpdisk.basePath = __dirname;
  try {
    tmpdisk.serve(function() {
      var reqData = ' plus some more text.';
      var req = http.request({
        hostname: 'localhost',
        port: 8089,
        path: '/',
        method: 'POST',
        headers: {
          'Content-Type': 'plain/text',
          'Content-Length': reqData.length,
          'Tmpdisk-Request-File-Path': 'test.txt',
          'Tmpdisk-Request-File-Encoding': 'utf8',
          'Tmpdisk-Request-Type': 'append'
        }
      }, function(res) {
        var data = '';

        res.on('data', function(chunk) {
          data += String(chunk);
        });

        res.on('end', function() {
          if (res.statusCode === 500 || res.statusCode === 403 || res.statusCode === 400) {
            tmpdisk.close();
            return t.fail('failed to perform server request.');
          }

          if (String(fs.readFileSync(__dirname + '/test.txt')) !== 'this is some *new* test text. plus some more text.') {
            tmpdisk.close();
            return t.fail('new file contents recieved are not equal to what they should be.');
          }

          tmpdisk.close();
          t.pass('successfully appended file.');
        });
      });
      req.on('error', function(e) {
        t.fail('failed to access server.');
      });
      req.write(reqData);
      req.end();
    });
  } catch(e) {
    t.fail('failed to create server.');
  }
});

test('can delete file?', function(t) {
  t.plan(1);
  tmpdisk.port = 8089;
  tmpdisk.basePath = __dirname;
  try {
    tmpdisk.serve(function() {
      var reqData = 'test.txt';
      var req = http.request({
        hostname: 'localhost',
        port: 8089,
        path: '/',
        method: 'POST',
        headers: {
          'Content-Type': 'plain/text',
          'Content-Length': reqData.length,
          'Tmpdisk-Request-Type': 'delete'
        }
      }, function(res) {
        var data = '';

        res.on('data', function(chunk) {
          data += String(chunk);
        });

        res.on('end', function() {
          if (res.statusCode === 500 || res.statusCode === 403 || res.statusCode === 400) {
            tmpdisk.close();
            return t.fail('failed to perform server request.');
          }

          fs.stat(__dirname + '/test.txt', function(err, stats) {
            if (!err) {
              tmpdisk.close();
              return t.fail('failed to delete file.');
            }

            tmpdisk.close();
            t.pass('successfully deleted file.');
          });
        });
      });
      req.on('error', function(e) {
        t.fail('failed to access server.');
      });
      req.write(reqData);
      req.end();
    });
  } catch(e) {
    t.fail('failed to create server.');
  }
});

test('can delete folder?', function(t) {
  t.plan(1);
  tmpdisk.port = 8089;
  tmpdisk.basePath = __dirname;
  fs.unlinkSync(__dirname + '/testFolder/blah.txt');
  try {
    tmpdisk.serve(function() {
      var reqData = 'testFolder';
      var req = http.request({
        hostname: 'localhost',
        port: 8089,
        path: '/',
        method: 'POST',
        headers: {
          'Content-Type': 'plain/text',
          'Content-Length': reqData.length,
          'Tmpdisk-Request-Type': 'delete'
        }
      }, function(res) {
        var data = '';

        res.on('data', function(chunk) {
          data += String(chunk);
        });

        res.on('end', function() {
          if (res.statusCode === 500 || res.statusCode === 403 || res.statusCode === 400) {
            tmpdisk.close();
            return t.fail('failed to perform server request.');
          }

          fs.stat(__dirname + '/testFolder', function(err, stats) {
            if (!err) {
              tmpdisk.close();
              return t.fail('failed to delete folder.');
            }

            tmpdisk.close();
            t.pass('successfully deleted folder.');
          });
        });
      });
      req.on('error', function(e) {
        t.fail('failed to access server.');
      });
      req.write(reqData);
      req.end();
    });
  } catch(e) {
    t.fail('failed to create server.');
  }
});

test('does throw bad request?', function(t) {
  t.plan(1);
  tmpdisk.port = 8089;
  tmpdisk.basePath = __dirname;
  try {
    tmpdisk.serve(function() {
      var reqData = 'testFolder';
      var req = http.request({
        hostname: 'localhost',
        port: 8089,
        path: '/',
        method: 'POST',
        headers: {
          'Content-Type': 'plain/text',
          'Content-Length': reqData.length,
          'Tmpdisk-Request-Type': 'blah'
        }
      }, function(res) {
        var data = '';

        res.on('data', function(chunk) {
          data += String(chunk);
        });

        res.on('end', function() {
          if (res.statusCode === 400) {
            tmpdisk.close();
            t.pass('succeeded to throw bad request.');
          } else {
            tmpdisk.close();
            t.fail('failed to throw bad request.');
          }
        });
      });
      req.on('error', function(e) {
        t.fail('failed to access server.');
      });
      req.write(reqData);
      req.end();
    });
  } catch(e) {
    t.fail('failed to create server.');
  }
});

test('does throw forbidden error?', function(t) {
  t.plan(1);
  tmpdisk.port = 8089;
  tmpdisk.basePath = __dirname;
  try {
    tmpdisk.serve(function() {
      var reqData = '../';
      var req = http.request({
        hostname: 'localhost',
        port: 8089,
        path: '/',
        method: 'POST',
        headers: {
          'Content-Type': 'plain/text',
          'Content-Length': reqData.length,
          'Tmpdisk-Request-Type': 'read'
        }
      }, function(res) {
        var data = '';

        res.on('data', function(chunk) {
          data += String(chunk);
        });

        res.on('end', function() {
          if (res.statusCode === 403) {
            tmpdisk.close();
            t.pass('succeeded to throw forbidden error.');
          } else {
            tmpdisk.close();
            t.fail('failed to throw forbidden error.');
          }
        });
      });
      req.on('error', function(e) {
        t.fail('failed to access server.');
      });
      req.write(reqData);
      req.end();
    });
  } catch(e) {
    t.fail('failed to create server.');
  }
});

test('does throw server error?', function(t) {
  t.plan(1);
  tmpdisk.port = 8089;
  tmpdisk.basePath = __dirname;
  try {
    tmpdisk.serve(function() {
      var reqData = 'testFolder/does_not_exist.txt';
      var req = http.request({
        hostname: 'localhost',
        port: 8089,
        path: '/',
        method: 'POST',
        headers: {
          'Content-Type': 'plain/text',
          'Content-Length': reqData.length,
          'Tmpdisk-Request-Type': 'read'
        }
      }, function(res) {
        var data = '';

        res.on('data', function(chunk) {
          data += String(chunk);
        });

        res.on('end', function() {
          if (res.statusCode === 500) {
            tmpdisk.close();
            t.pass('succeeded to throw server error.');
          } else {
            tmpdisk.close();
            t.fail('failed to throw server error.');
          }
        });
      });
      req.on('error', function(e) {
        t.fail('failed to access server.');
      });
      req.write(reqData);
      req.end();
    });
  } catch(e) {
    t.fail('failed to create server.');
  }
});

test('does throw bad request on trying to write to a directory?', function(t) {
  t.plan(1);
  tmpdisk.port = 8089;
  tmpdisk.basePath = __dirname;
  try {
    fs.mkdirSync(__dirname + '/testFolder');
    fs.writeFileSync(__dirname + '/testFolder/blah.txt', 'blah');
  } catch(e) {
    // do nothing
  }
  try {
    tmpdisk.serve(function() {
      var reqData = 'this_data_shouldn\'t_be_written';
      var req = http.request({
        hostname: 'localhost',
        port: 8089,
        path: '/',
        method: 'POST',
        headers: {
          'Content-Type': 'plain/text',
          'Content-Length': reqData.length,
          'Tmpdisk-Request-File-Path': 'testFolder/',
          'Tmpdisk-Request-File-Encoding': 'utf8',
          'Tmpdisk-Request-Type': 'write'
        }
      }, function(res) {
        var data = '';

        res.on('data', function(chunk) {
          data += String(chunk);
        });

        res.on('end', function() {
          if (res.statusCode === 400) {
            tmpdisk.close();
            t.pass('succeeded to throw bad request.');
          } else {
            tmpdisk.close();
            t.fail('failed to throw bad request.');
          }
        });
      });
      req.on('error', function(e) {
        t.fail('failed to access server.');
      });
      req.write(reqData);
      req.end();
    });
  } catch(e) {
    t.fail('failed to create server.');
  }
});

test('does throw bad request on trying to append to a directory?', function(t) {
  t.plan(1);
  tmpdisk.port = 8089;
  tmpdisk.basePath = __dirname;
  try {
    tmpdisk.serve(function() {
      var reqData = 'this_data_shouldn\'t_be_appended';
      var req = http.request({
        hostname: 'localhost',
        port: 8089,
        path: '/',
        method: 'POST',
        headers: {
          'Content-Type': 'plain/text',
          'Content-Length': reqData.length,
          'Tmpdisk-Request-File-Path': 'testFolder/',
          'Tmpdisk-Request-File-Encoding': 'utf8',
          'Tmpdisk-Request-Type': 'append'
        }
      }, function(res) {
        var data = '';

        res.on('data', function(chunk) {
          data += String(chunk);
        });

        res.on('end', function() {
          if (res.statusCode === 400) {
            tmpdisk.close();
            t.pass('succeeded to throw bad request.');
          } else {
            tmpdisk.close();
            t.fail('failed to throw bad request.');
          }
        });
      });
      req.on('error', function(e) {
        t.fail('failed to access server.');
      });
      req.write(reqData);
      req.end();
    });
  } catch(e) {
    t.fail('failed to create server.');
  }
});

test('does create file if it doesn\'t exist (on write)?', function(t) {
  t.plan(1);
  tmpdisk.port = 8089;
  tmpdisk.basePath = __dirname;
  try {
    tmpdisk.serve(function() {
      var reqData = 'this is some more test text.';
      var req = http.request({
        hostname: 'localhost',
        port: 8089,
        path: '/',
        method: 'POST',
        headers: {
          'Content-Type': 'plain/text',
          'Content-Length': reqData.length,
          'Tmpdisk-Request-File-Path': 'this_is_new.txt',
          'Tmpdisk-Request-File-Encoding': 'utf8',
          'Tmpdisk-Request-Type': 'write'
        }
      }, function(res) {
        var data = '';

        res.on('data', function(chunk) {
          data += String(chunk);
        });

        res.on('end', function() {
          if (res.statusCode === 500 || res.statusCode === 403 || res.statusCode === 400) {
            tmpdisk.close();
            return t.fail('failed to perform server request.');
          }

          fs.stat(__dirname + '/this_is_new.txt', function(err, stats) {
            if (err) {
              tmpdisk.close();
              return t.fail('failed to create file.');
            }

            if (String(fs.readFileSync(__dirname + '/this_is_new.txt')) !== 'this is some more test text.') {
              tmpdisk.close();
              return t.fail('created file contents recieved are not equal to what they should be.');
            }

            tmpdisk.close();
            t.pass('successfully created file.');
          });
        });
      });
      req.on('error', function(e) {
        t.fail('failed to access server.');
      });
      req.write(reqData);
      req.end();
    });
  } catch(e) {
    t.fail('failed to create server.');
  }
});

test('cleanup', function(t) {
  t.plan(1);
  try {
    fs.writeFileSync(__dirname + '/test.txt', 'this is some test text.');
    fs.unlinkSync(__dirname + '/this_is_new.txt');
    t.pass('done cleaning up.');
  } catch(e) {
    t.fail('failed to clean up.');
  }
});
